import { Injectable, Inject } from '@nestjs/common';
import { User } from './entities/user.entity';
import { CreateUserDto, UpdateUserDto } from './dto';
import { USER_REPOSITORY, WALLET_REPOSITORY } from '../../core/constants';
import * as argon from 'argon2';
import * as _ from 'lodash';
import { UUID } from 'crypto';
import { Wallet } from '../wallets/entities/wallet.entity';
import { CreateWalletDto } from '../wallets/dto';

@Injectable()
export class UsersService {
  constructor(
    @Inject(USER_REPOSITORY) private readonly userRepository: typeof User,
    @Inject(WALLET_REPOSITORY) private readonly walletRepository: typeof Wallet,
  ) {}

  async findAll() {
    return await this.userRepository.findAll();
  }

  async findOne(id: UUID) {
    return await this.userRepository.findAll({ where: { uuid: id } });
  }

  async create(createUserDto: CreateUserDto) {
    const hashedPassword = await argon.hash(createUserDto.password);
    createUserDto.password = hashedPassword;

    const newUser = await this.userRepository.create(createUserDto);

    await this.createWallet({ userId: newUser.id, amount: 0 });

    return _.pick(newUser, [
      'firstName',
      'lastName',
      'email',
      'uuid',
      'createdAt',
      'updatedAt',
    ]);
  }

  async update(id: UUID, updateUserDto: UpdateUserDto) {
    const [numberOfAffectedRows, [updatedUser]] =
      await this.userRepository.update(
        { ...updateUserDto },
        { where: { uuid: id }, returning: true },
      );
    return { numberOfAffectedRows, updatedUser };
  }

  async remove(id: UUID) {
    const deletedUser = await this.findOne(id);

    await this.userRepository.destroy({ where: { uuid: id } });

    return deletedUser;
  }

  async findOneByEmail(email: string): Promise<User> {
    return await this.userRepository.findOne<User>({ where: { email } });
  }

  private async createWallet(createWalletDto: CreateWalletDto) {
    await this.walletRepository.create(createWalletDto);
  }
}
